"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const url = require("url");
const chalk_1 = require("chalk");
const eventemitter2_1 = require("eventemitter2");
const lodash_1 = require("lodash");
const debug_1 = require("./utils/debug");
const config_hints_1 = require("./config/config-hints");
const logger = require("./utils/logging");
const hint_context_1 = require("./hint-context");
const hintscope_1 = require("./enums/hintscope");
const debug = debug_1.debug(__filename);
class Engine extends eventemitter2_1.EventEmitter2 {
    constructor(config, resources) {
        super({
            delimiter: '::',
            maxListeners: 0,
            wildcard: true
        });
        this.browserslist = [];
        this._timeout = 60000;
        debug('Initializing hint engine');
        this._timeout = config.hintsTimeout;
        this.messages = [];
        this.browserslist = config.browserslist;
        this.ignoredUrls = config.ignoredUrls;
        const Connector = resources.connector;
        const connectorId = config.connector && config.connector.name;
        if (!Connector) {
            throw new Error(`Connector "${connectorId}" not found`);
        }
        this.connector = new Connector(this, config.connector && config.connector.options);
        this._formatters = resources.formatters.map((Formatter) => {
            return new Formatter();
        });
        this.parsers = resources.parsers.map((ParserConstructor) => {
            debug(`Loading parser`);
            return new ParserConstructor(this);
        });
        debug(`Parsers loaded: ${this.parsers.length}`);
        this.hints = new Map();
        const getHintConfig = (id) => {
            if (config.hints[id]) {
                return config.hints[id];
            }
            const hintEntries = Object.keys(config.hints);
            const idParts = id.split('/');
            const hintKey = hintEntries.find((entry) => {
                return idParts.every((idPart) => {
                    return entry.includes(idPart);
                });
            });
            return config.hints[hintKey || ''];
        };
        resources.hints.forEach((Hint) => {
            debug('Loading hints');
            const id = Hint.meta.id;
            const ignoreHint = (HintCtor) => {
                const ignoredConnectors = HintCtor.meta.ignoredConnectors || [];
                return (connectorId === 'local' && HintCtor.meta.scope === hintscope_1.HintScope.site) ||
                    (connectorId !== 'local' && HintCtor.meta.scope === hintscope_1.HintScope.local) ||
                    ignoredConnectors.includes(connectorId || '');
            };
            const hintOptions = getHintConfig(id);
            const severity = config_hints_1.getSeverity(hintOptions);
            if (ignoreHint(Hint)) {
                debug(`Hint "${id}" is disabled for the connector "${connectorId}"`);
                logger.log(chalk_1.default.yellow(`Warning: The hint "${id}" will be ignored for the connector "${connectorId}"`));
            }
            else if (severity) {
                const context = new hint_context_1.HintContext(id, this, severity, hintOptions, Hint.meta);
                const hint = new Hint(context);
                this.hints.set(id, hint);
            }
            else {
                debug(`Hint "${id}" is disabled`);
            }
        });
    }
    get pageDOM() {
        return this.connector.dom;
    }
    get pageContent() {
        return this.connector.html;
    }
    get pageHeaders() {
        return this.connector.headers;
    }
    get targetedBrowsers() {
        return this.browserslist;
    }
    get formatters() {
        return this._formatters;
    }
    get timeout() {
        return this._timeout;
    }
    isIgnored(urls, resource) {
        if (!urls) {
            return false;
        }
        return urls.some((urlIgnored) => {
            return urlIgnored.test(resource);
        });
    }
    onHintEvent(id, eventName, listener) {
        const that = this;
        const createEventHandler = (handler, hintId) => {
            return function (event) {
                const urlsIgnoredForAll = that.ignoredUrls.get('all') || [];
                const urlsIgnoredForHint = that.ignoredUrls.get(hintId) || [];
                const urlsIgnored = urlsIgnoredForHint.concat(urlsIgnoredForAll);
                if (that.isIgnored(urlsIgnored, event.resource)) {
                    return null;
                }
                return new Promise((resolve) => {
                    let immediateId;
                    const timeoutId = setTimeout(() => {
                        if (immediateId) {
                            clearImmediate(immediateId);
                            immediateId = null;
                        }
                        debug(`Hint ${hintId} timeout`);
                        resolve(null);
                    }, that._timeout);
                    immediateId = setImmediate(async () => {
                        const result = await handler(event, this.event);
                        if (timeoutId) {
                            clearTimeout(timeoutId);
                        }
                        resolve(result);
                    });
                });
            };
        };
        this.on(eventName, createEventHandler(listener, id));
    }
    fetchContent(target, headers) {
        return this.connector.fetchContent(target, headers);
    }
    evaluate(source) {
        return this.connector.evaluate(source);
    }
    async close() {
        await this.connector.close();
    }
    report(hintId, category, severity, sourceCode, location, message, resource) {
        const problem = {
            category,
            hintId,
            location: location || { column: -1, line: -1 },
            message,
            resource,
            severity,
            sourceCode
        };
        this.messages.push(problem);
    }
    clean(fileUrl) {
        const file = url.format(fileUrl);
        lodash_1.remove(this.messages, (message) => {
            return message.resource === file;
        });
    }
    clear() {
        this.messages = [];
    }
    async notify() {
        await this.emitAsync('print', this.messages);
    }
    async executeOn(target, options) {
        const start = Date.now();
        debug(`Starting the analysis on ${target.href}`);
        await this.connector.collect(target, options);
        debug(`Total runtime ${Date.now() - start}`);
        return this.messages;
    }
    querySelectorAll(selector) {
        return this.connector.querySelectorAll(selector);
    }
    emit(event, data) {
        return super.emit(event, data);
    }
    emitAsync(event, data) {
        const ignoredUrls = this.ignoredUrls.get('all') || [];
        if (this.isIgnored(ignoredUrls, data.resource)) {
            return Promise.resolve([]);
        }
        return super.emitAsync(event, data);
    }
    on(event, listener) {
        return super.on(event, listener);
    }
}
exports.Engine = Engine;
