"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ajv = require("ajv");
const lodash_1 = require("lodash");
const validator = new ajv({
    $data: true,
    allErrors: true,
    logger: false,
    schemaId: 'id',
    useDefaults: true,
    verbose: true
});
validator.addMetaSchema(require('ajv/lib/refs/json-schema-draft-04.json'));
var ErrorKeyword;
(function (ErrorKeyword) {
    ErrorKeyword["additionalProperties"] = "additionalProperties";
    ErrorKeyword["anyOf"] = "anyOf";
    ErrorKeyword["enum"] = "enum";
    ErrorKeyword["pattern"] = "pattern";
    ErrorKeyword["type"] = "type";
})(ErrorKeyword || (ErrorKeyword = {}));
const generateError = (type, action) => {
    return (error, errors) => {
        if (error.keyword !== type) {
            return null;
        }
        const property = error.dataPath.substr(1);
        return action(error, property, errors);
    };
};
const generateAdditionalPropertiesError = generateError(ErrorKeyword.additionalProperties, (error, property) => {
    const additionalProperty = error.params.additionalProperty;
    return `${property ? `'${property}' ` : ''}${property ? error.message : `${error.message && error.message[0].toLocaleUpperCase()}${error.message && error.message.substr(1)}`}. Additional property found '${additionalProperty}'.`;
});
const generateEnumError = generateError(ErrorKeyword.enum, (error, property) => {
    const allowedValues = error.params.allowedValues;
    return `'${property}' ${error.message} '${allowedValues.join(', ')}'. Value found '${error.data}'`;
});
const generatePatternError = generateError(ErrorKeyword.pattern, (error, property) => {
    return `'${property}' ${error.message && error.message.replace(/"/g, '\'')}. Value found '${error.data}'`;
});
const generateTypeError = generateError(ErrorKeyword.type, (error, property) => {
    return `'${property}' ${error.message && error.message.replace(/"/g, '\'')}.`;
});
const generateAnyOfError = generateError(ErrorKeyword.anyOf, (error, property, errors) => {
    const otherErrors = lodash_1.without(errors, error);
    const results = otherErrors.map((otherError) => {
        return generate(otherError);
    });
    return results.join(' or ');
});
const errorGenerators = [generateAdditionalPropertiesError, generateEnumError, generatePatternError, generateTypeError, generateAnyOfError];
const generate = (error, errors) => {
    return errorGenerators.reduce((message, generator) => {
        const newErrorMessage = generator(error, errors);
        if (newErrorMessage) {
            return newErrorMessage;
        }
        return message;
    }, error.message || '');
};
const prettify = (errors) => {
    const grouped = lodash_1.groupBy(errors, 'dataPath');
    const result = lodash_1.reduce(grouped, (allMessages, groupErrors) => {
        groupErrors.forEach((error) => {
            allMessages.push(generate(error, groupErrors) || '');
        });
        return allMessages;
    }, []);
    return result;
};
const errorWithLocation = (error, getLocation) => {
    let path = error.dataPath;
    const additionalProperty = error.params && error.params.additionalProperty;
    if (additionalProperty) {
        path = path ? `${path}.${additionalProperty}` : additionalProperty;
    }
    return { ...error, location: getLocation(path) || undefined };
};
exports.validate = (schema, json, getLocation) => {
    const data = lodash_1.cloneDeep(json);
    const validateFunction = validator.compile(schema);
    const valid = validateFunction(data);
    let errors = validateFunction.errors || [];
    if (errors && getLocation) {
        errors = errors.map((e) => {
            return errorWithLocation(e, getLocation);
        });
    }
    return {
        data,
        errors,
        prettifiedErrors: prettify(errors),
        valid
    };
};
